program SplSlntStk
  ! Slant stack with spline interpolation

  use sep, only: sep_init, sep_close, sep_read, sep_write, to_history
  use ooio, only: axis, readcharpar, readaxis, writeaxis, chk_dim_corresp

  implicit none
  real, allocatable, dimension (:,:) :: U, W
  real, allocatable, dimension (:)   :: a!, b, c, d
  real                               :: b, tau
  type(axis)                         :: x, y, t, v
  integer                            :: i, it, ix, iv

  call sep_init()
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Read/write axes
  t = readaxis(1) ! TIME
  x = readaxis(2) ! OFFSET
  y = readaxis(3) ! MIDPOINT

  v%o = 1.5
  v%n = 200
  v%d = 0.0135

  call writeaxis(v, 2, axisname='V(km/s)')
  call sep_close()
  allocate(t%all(t%n),x%all(x%n),v%all(v%n))

  do it=1, t%n
     t%all(it) = t%o + (it-1) * t%d
  end do

  do ix=1, x%n
     x%all(ix) = x%o + (ix-1) * x%d
  end do

  do iv=1, v%n
     v%all(iv) = v%o + (iv-1) * v%d
  end do
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  allocate(U(t%n,x%n), W(t%n,v%n),a(t%n)); U=0; W=0; a=0; b=0

  main_loop:do i=1,y%n
     call sep_read(U)

     do ix=1, x%n
        call spline    (t%n, t%all, U(:,ix), a)
        do it=1,t%n
           do iv=1,v%n
              tau = t%all(it) + x%all(ix) / v%all(iv)
              if(tau<=t%all(t%n)) call speval (t%n, t%all, U(:,ix), a , tau, b)
              W(it,iv) = W(it,iv) + b
           end do
        end do
        write(0,*) 'offset',ix,'out of',x%n
     end do

     call sep_write(W)
     write(0,*) 'processed midpoint',i,'out of',y%n
  end do main_loop

  deallocate(U, W, a, t%all, x%all, v%all) 
end program SplSlntStk
