program T2X2
  ! Program to transform CMP gathers in a (time=t, offset=x, midpoint=y) cube (2-D line) from (t,x) to (t^2,x^2) so that hyperbolas become straight lines.

  use sep, only: sep_init, sep_close, sep_read, sep_write, to_history
  use ooio, only: axis, readcharpar, readaxis, writeaxis, chk_dim_corresp

  implicit none
  real, allocatable, dimension (:,:) :: U
  real, allocatable, dimension (:)   :: a, b, c, d
  type(axis)                         :: x, y, t, x2, t2
  character(len=6)                   :: interp_type
  integer                            :: i, it, ix

  call sep_init()
  interp_type = readcharpar('interp_type',(/'linear','spline'/), 1)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Read/write axes
  t = readaxis(1) ! TIME
  x = readaxis(2) ! OFFSET
  y = readaxis(3) ! MIDPOINT

  ! Generating the T^2 axis
  allocate(t%all(t%n),t2%all(t%n))
  t2%n = t%n
  t2%o = t%o**2
  do i=1, t%n
     t%all(i) = t%o + (i-1)*t%d
  end do
  t2%all = t%all**2
  t2%d = ( t2%all(t%n) - t2%o ) / ( t%n - 1 )

  ! Generating the X^2 axis
  allocate(x%all(x%n),x2%all(x%n))
  x2%n = x%n
  x2%o = x%o**2
  do i=1, x%n
     x%all(i) = x%o + (i-1)*x%d
  end do
  x2%all = x%all**2
  x2%d = ( x2%all(x%n) - x2%o ) / ( x%n - 1 )

  call writeaxis(x2, 2, axisname='x^2')
  call writeaxis(t2, 1, axisname='t^2')

  call sep_close()
  allocate(U(t%n,x%n),a(t%n),b(x%n),c(t%n),d(x%n)); U=0; a=0; b=0; c=0; d=0

  main_loop:do i=1,y%n
     call sep_read(U)

     t_loop:do it=1, t%n
        call spline    (x2%n, x2%all, U(it,:), b)
        x_in_t_loop: do ix=1, x2%n
           call speval (x2%n, x2%all, U(it,:), b , x2%o + (ix-1)* x2%d, d(ix))
        end do x_in_t_loop
        U(it,:) = d
     end do t_loop

     x_loop:do ix=1, x%n
        call spline    (t2%n, t2%all, U(:,ix), a)
        t_in_x_loop: do it=1, t2%n
           call speval (t2%n, t2%all, U(:,ix), a , t2%o + (it-1)* t2%d, c(it))
        end do t_in_x_loop
        U(:,ix) = c
     end do x_loop

     call sep_write(U)
     write(0,*) 'processed midpoint',i,'out of',y%n
  end do main_loop

deallocate(x%all, x2%all, t%all, t2%all, U, a, b, c, d) 
end program T2X2
